@@ -46,6 +46,7 @@ pub enum Operations {
     WeakEqual,
     Peek(usize),
     Poke(usize),
+    PeekSize,
     PokeSize,
     PopCount,
     Ceiling,
@@ -79,6 +80,8 @@ pub enum Operations {
     GoTo,
     ToDo,
     NoOperation,
+    Print, // Tool for cli hooks
+    Constrain,
 
     // flag ops
     Zero,
@@ -134,6 +137,7 @@ impl Operations {
             "--" => Operations::Decrement,
             "=" => Operations::Equal,
             ":=" => Operations::WeakEqual,
+            "[]" => Operations::PeekSize,
             "[1]" => Operations::Peek(1),
             "[2]" => Operations::Peek(2),
             "[4]" => Operations::Peek(4),
@@ -176,6 +180,10 @@ impl Operations {
             "GOTO" => Operations::GoTo,
             "TODO" => Operations::ToDo,
             "" => Operations::NoOperation,
+
+            // hax for use in the cli / plugin
+            "." => Operations::Print,
+            "_" => Operations::Constrain,
         
             "$z" => Operations::Zero,
             "$c" => Operations::Carry,
@@ -195,7 +203,7 @@ impl Operations {
 
 #[inline]
 pub fn get_size(state: &mut State) -> u32 {
-    let item = state.stack.pop().unwrap();
+    let item = state.stack.pop().unwrap_or_default();
 
     let sz = match &item {
         StackItem::StackValue(_val) => {
@@ -213,7 +221,7 @@ pub fn get_size(state: &mut State) -> u32 {
 
 #[inline]
 pub fn pop_value(state: &mut State, set_size: bool, sign_ext: bool) -> Value {
-    let item = state.stack.pop().unwrap();
+    let item = state.stack.pop().unwrap_or_default();
 
     let value = match item {
         StackItem::StackValue(val) => {
@@ -253,7 +261,7 @@ pub fn pop_value(state: &mut State, set_size: bool, sign_ext: bool) -> Value {
 
 #[inline]
 pub fn pop_stack_value(state: &mut State, stack: &mut Vec<StackItem>, set_size: bool, sign_ext: bool) -> Value {
-    let item = stack.pop().unwrap();
+    let item = stack.pop().unwrap_or_default();
 
     let value = match item {
         StackItem::StackValue(val) => {
@@ -271,7 +279,7 @@ pub fn pop_stack_value(state: &mut State, stack: &mut Vec<StackItem>, set_size:
     match &value {
         Value::Concrete(_v, _t) => value,
         Value::Symbolic(ov, t) => {
-            if ov.is_const() {
+            if ov.is_const() && ov.get_width() <= 64 {
                 Value::Concrete(ov.as_u64().unwrap(), *t)
             } else {
                 let v = ov; //state.translate(&ov).unwrap();
@@ -299,16 +307,12 @@ pub fn push_value(state: &mut State, value: Value) {
 pub fn pop_concrete(state: &mut State, set_size: bool, sign_ext: bool) -> u64 {
     let value = pop_value(state, set_size, sign_ext);
 
-    match value {
+    match &value {
         Value::Concrete(val, _t) => {
-            val
+            *val
         },
-        Value::Symbolic(val, _t) => {
-            let solution = val.get_a_solution().as_u64().unwrap();
-            let sol_bv = state.bvv(solution, 64);
-            let a = val._eq(&sol_bv);
-            state.solver.assert(&a);
-            solution
+        Value::Symbolic(_val, _t) => {
+            state.solver.evalcon_to_u64(&value).unwrap()
         }
     }
 }
@@ -407,9 +411,7 @@ pub fn do_operation(state: &mut State, operation: Operations) {
         Operations::Compare => {
             let arg1 = pop_value(state, true, false);
             let arg2 = pop_value(state, false, false);
-
             state.esil.current = arg1.to_owned() - arg2;
-            //println!("cmp {:?}", state.esil.current);
             state.esil.previous = arg1;
         },
         Operations::LessThan => {
@@ -588,9 +590,20 @@ pub fn do_operation(state: &mut State, operation: Operations) {
             state.esil.previous = addr;
             state.esil.last_sz = 8*n;
         },
+        Operations::PeekSize => {
+            let n = pop_concrete(state, false, false) as usize;
+            let addr = pop_value(state, false, false);
+
+            let val = state.memory_read_value(&addr, n);
+            state.esil.current = val.to_owned();
+            push_value(state, val);
+
+            state.esil.previous = addr;
+            state.esil.last_sz = 8*n;
+        },
         Operations::PokeSize => {
+            let n = pop_concrete(state, false, false) as usize;
             let addr = pop_value(state, false, false);
-            let n = (get_size(state)/8) as usize;
             let value = pop_value(state, false, false);
 
             if let Some(cond) = &state.condition.to_owned() {
@@ -630,52 +643,45 @@ pub fn do_operation(state: &mut State, operation: Operations) {
                     for i in 0..val.get_width() {
                         sym_val = sym_val.add(&val.slice(i+1, i).uext(63));
                     }
-                    let value = Value::Symbolic(sym_val, t);
-                    push_value(state, value);
+                    push_value(state, Value::Symbolic(sym_val, t));
                 }
             }
         }, 
         Operations::Ceiling => {
             let t = get_stack_taint(state, 1);
             let arg1 = pop_double(state);
-            let value = Value::Concrete(f64::to_bits(arg1.ceil()), t); 
-            push_value(state, value);
+            push_value(state, Value::Concrete(f64::to_bits(arg1.ceil()), t));
         },
         Operations::Floor => {
             let t = get_stack_taint(state, 1);
             let arg1 = pop_double(state);
-            let value = Value::Concrete(f64::to_bits(arg1.floor()), t);
-            push_value(state, value);
+            push_value(state, Value::Concrete(f64::to_bits(arg1.floor()), t));
         },
         Operations::Round => {
             let t = get_stack_taint(state, 1);
             let arg1 = pop_double(state);
-            let value = Value::Concrete(f64::to_bits(arg1.round()), t);
-            push_value(state, value);
+            push_value(state, Value::Concrete(f64::to_bits(arg1.round()), t));
         },
         Operations::SquareRoot => {
             let t = get_stack_taint(state, 1);
             let arg1 = pop_double(state);
-            let value = Value::Concrete(f64::to_bits(arg1.sqrt()), t);
-            push_value(state, value);
+            push_value(state, Value::Concrete(f64::to_bits(arg1.sqrt()), t));
         },
         Operations::DoubleToInt => {
             let t = get_stack_taint(state, 1);
             let arg1 = pop_double(state);
-            let value = Value::Concrete(arg1 as u64, t);
-            push_value(state, value);
+            push_value(state, Value::Concrete(arg1 as u64, t));
         },
         Operations::SignedToDouble => {
             let t = get_stack_taint(state, 1);
             let arg1 = pop_concrete(state, false, true);
-            let value = Value::Concrete(f64::to_bits(arg1 as i64 as f64), t); //hmm
+            let value = Value::Concrete(f64::to_bits(arg1 as i64 as f64), t);
             push_value(state, value);
         },
         Operations::UnsignedToDouble => {
             let t = get_stack_taint(state, 1);
             let arg1 = pop_concrete(state, false, false);
-            let value = Value::Concrete(f64::to_bits(arg1 as f64), t); 
-            push_value(state, value);
+            push_value(state, Value::Concrete(f64::to_bits(arg1 as f64), t));
         },
         Operations::FloatToDouble => {
             let val = pop_value(state, false, false);
@@ -723,27 +729,23 @@ pub fn do_operation(state: &mut State, operation: Operations) {
             let t = get_stack_taint(state, 2);
             let arg1 = pop_double(state);
             let arg2 = pop_double(state);
-            let value = Value::Concrete((arg1 - arg2 == 0.0) as u64, t);
-            push_value(state, value);
+            push_value(state, Value::Concrete((arg1 - arg2 == 0.0) as u64, t));
         },
         Operations::FloatLessThan => {
             let t = get_stack_taint(state, 2);
             let arg1 = pop_double(state);
             let arg2 = pop_double(state);
-            let value = Value::Concrete((arg1 < arg2) as u64, t);
-            push_value(state, value);
+            push_value(state, Value::Concrete((arg1 < arg2) as u64, t));
         },
         Operations::NaN => {
             let t = get_stack_taint(state, 1);
             let arg1 = pop_double(state);
-            let value = Value::Concrete(arg1.is_nan() as u64, t);
-            push_value(state, value);
+            push_value(state, Value::Concrete(arg1.is_nan() as u64, t));
         },
         Operations::FloatNegate => {
             let t = get_stack_taint(state, 1);
             let arg1 = pop_double(state);
-            let value = Value::Concrete(f64::to_bits(-arg1), t);
-            push_value(state, value);
+            push_value(state, Value::Concrete(f64::to_bits(-arg1), t));
         },
         Operations::Swap => {
             let arg1 = state.stack.pop().unwrap();
@@ -779,8 +781,29 @@ pub fn do_operation(state: &mut State, operation: Operations) {
         Operations::Break => {},
         Operations::Repeat => {},
         Operations::GoTo => {},
-        Operations::ToDo => {},
         Operations::NoOperation => {},
+
+        Operations::Print => {
+            let value = pop_value(state, false, false);
+            if let Some(bv) = state.solver.eval_to_bv(&value) {
+                let ip = state.registers.get_pc().as_u64().unwrap_or_default();
+                if let Some(string) = state.evaluate_string(&bv) {
+                    println!("{:016x}: {:?} {:?}", ip, bv, string);
+                } else {
+                    println!("{:016x}: {:?}", ip, bv);
+                }
+            }
+        },
+        Operations::Constrain => {
+            let value = pop_value(state, false, false);
+            state.assert_value(&value);
+        },
+
+        Operations::ToDo => {
+            if state.strict {
+                unimplemented!();
+            }
+        },
     
         Operations::Zero => {
             let cur = &state.esil.current;
@@ -854,6 +877,10 @@ pub fn do_operation(state: &mut State, operation: Operations) {
         Operations::R => {
             push_value(state, Value::Concrete(64 >> 3, 0));
         },
-        Operations::Unknown => {}
+        Operations::Unknown => {
+            if state.strict {
+                panic!("Encountered an unknown word!");
+            }
+        }
     }
 }
